@@ -1,4 +1,4 @@
-################################################################################
+############################################################################## #
 # Copyright (C) 2019 by 52 North                                               #
 # Initiative for Geospatial Open Source Software GmbH                          #
 #                                                                              #
@@ -22,10 +22,11 @@
 # visit the Free Software Foundation web page, http://www.fsf.org.             #
 #                                                                              #
 # Author: Daniel Nuest (daniel.nuest@uni-muenster.de)                          #
+#         Eike Hinderk Jürrens (e.h.juerrens@52north.org)                      #
 # Created: 2010-06-18                                                          #
 # Project: sos4R - https://github.com/52North/sos4R                            #
 #                                                                              #
-################################################################################
+############################################################################## #
 
 #
 #
@@ -98,11 +99,11 @@ setClass("SweTextBlock",
          }
 )
 
-
-# SWE 2.0
-
+#
+# SWE 2.0 ----
 #
 #
+# class SweTextEncoding ----
 #
 setClass("SweTextEncoding",
          representation(tokenSeparator = "character",
@@ -120,8 +121,38 @@ setClass("SweTextEncoding",
          }
 )
 
+.print.SweTextEncoding <- function(x, ...) {
+  cat(.toString.SweTextEncoding(x, ...), "\n")
+  invisible(x)
+}
 
-################################################################################
+.toString.SweTextEncoding <- function(x, ...) {
+  s <- paste("Object of class SweTextEncoding",
+              " '",
+              x@tokenSeparator,
+              " ",
+              x@blockSeparator,
+              " ",
+              x@decimalSeparator,
+              "'; id: ",
+              x@id,
+              sep = "")
+  return(s)
+}
+
+setMethod("print", "SweTextEncoding", function(x, ...) .print.SweTextEncoding(x, ...))
+setMethod("show", "SweTextEncoding", function(object) .print.SweTextEncoding(object))
+setMethod("toString", "SweTextEncoding", function(x, ...) .toString.SweTextEncoding(x, ...))
+
+SweTextEncoding <- function(tokenSeparator, blockSeparator, decimalSeparator = as.character(NA),
+                            id = as.character(NA)) {
+  new("SweTextEncoding", tokenSeparator = tokenSeparator,
+      blockSeparator = blockSeparator,
+      decimalSeparator = decimalSeparator, id = id)
+}
+
+
+############################################################################## #
 #
 # Other SOS related SWE elements, like DataRecord, SimpleDataRecord, values, and
 # so forth are parsed directly into R structures.
